@@ -71,8 +71,13 @@ pairindextypes = {
 def _parse_annotation(annotation: str) -> List[Node]:
     """Parse type annotation."""
     def make_xref(text: str) -> addnodes.pending_xref:
+        if text == 'None':
+            reftype = 'obj'
+        else:
+            reftype = 'class'
+
         return pending_xref('', nodes.Text(text),
-                            refdomain='py', reftype='class', reftarget=text)
+                            refdomain='py', reftype=reftype, reftarget=text)
 
     def unparse(node: ast.AST) -> List[Node]:
         if isinstance(node, ast.Attribute):
@@ -1311,7 +1316,7 @@ def builtin_resolver(app: Sphinx, env: BuildEnvironment,
 
     if node.get('refdomain') != 'py':
         return None
-    elif node.get('reftype') == 'obj' and node.get('reftarget') == 'None':
+    elif node.get('reftype') in ('class', 'obj') and node.get('reftarget') == 'None':
         return contnode
     elif node.get('reftype') in ('class', 'exc'):
         reftarget = node.get('reftarget')
