@@ -17,7 +17,7 @@ import typing
 import warnings
 from functools import partial, partialmethod
 from inspect import (  # NOQA
-    Parameter, isclass, ismethod, ismethoddescriptor, unwrap
+    Parameter, isclass, ismethod, ismethoddescriptor
 )
 from io import StringIO
 from typing import Any, Callable, Mapping, List, Tuple
@@ -116,6 +116,15 @@ def getargspec(func: Callable) -> Any:
                                kwonlyargs, kwdefaults, annotations)
 
 
+def unwrap(obj: Any) -> Any:
+    """Get an original object from wrapped object (wrapped functions)."""
+    try:
+        return inspect.unwrap(obj)
+    except ValueError:
+        # might be a mock object
+        return obj
+
+
 def unwrap_all(obj: Any) -> Any:
     """
     Get an original object from wrapped object (unwrapping partials, wrapped
@@ -217,7 +226,7 @@ def isattributedescriptor(obj: Any) -> bool:
         return True
     elif isdescriptor(obj):
         # non data descriptor
-        unwrapped = inspect.unwrap(obj)
+        unwrapped = unwrap(obj)
         if isfunction(unwrapped) or isbuiltin(unwrapped) or inspect.ismethod(unwrapped):
             # attribute must not be either function, builtin and method
             return False
